/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.api.connect.world;

import org.jetbrains.annotations.ApiStatus.Experimental;
import grondag.xm.api.modelstate.primitive.PrimitiveState;
import net.minecraft.class_2338;
import net.minecraft.class_2680;

// For masonry, true result means border IS present
@Experimental
public class MasonryHelper implements BlockTest<PrimitiveState> {
	private MasonryHelper() {
	}

	private static final ThreadLocal<MasonryHelper> POOL = ThreadLocal.withInitial(MasonryHelper::new);

	public static BlockTest<PrimitiveState> wrap(BlockTest<PrimitiveState> test) {
		final MasonryHelper result = POOL.get();
		result.test = test;
		return result;
	}

	private BlockTest<PrimitiveState> test;

	@Override
	public boolean apply(BlockTestContext<PrimitiveState> context) {
		if (context.fromModelState() == null) {
			return false;
		}

		final class_2680 toBlockState = context.toBlockState();
		final class_2338 toPos = context.toPos();

		// if not a sibling, mortar if against full opaque
		if (!test.apply(context)) {
			return toBlockState.method_26216(context.world(), toPos);
		}

		final class_2338 fromPos = context.fromPos();

		// between siblings, only mortar on three sides of cube
		// (other sibling will do the mortar on other sides)
		return (toPos.method_10263() == fromPos.method_10263() + 1 || toPos.method_10264() == fromPos.method_10264() - 1 || toPos.method_10260() == fromPos.method_10260() + 1);
	}
}
