/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.api.modelstate;

import java.util.List;
import java.util.Random;
import java.util.function.Consumer;

import org.jetbrains.annotations.ApiStatus.Experimental;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_2350;
import net.minecraft.class_2487;
import net.minecraft.class_2540;
import net.minecraft.class_2680;
import net.minecraft.class_777;
import io.vram.frex.api.buffer.QuadSink;
import io.vram.frex.api.model.BlockModel.BlockInputContext;
import io.vram.frex.api.model.ItemModel.ItemInputContext;

import grondag.xm.api.mesh.polygon.Polygon;
import grondag.xm.api.paint.PaintIndex;
import grondag.xm.primitive.ModelPrimitiveRegistryImpl;

@Experimental
public interface ModelState {
	MutableModelState mutableCopy();

	/**
	 * Persisted but not part of hash nor included in equals comparison. If true,
	 * refreshFromWorldState does nothing.
	 */
	boolean isStatic();

	boolean isImmutable();

	ModelState toImmutable();

	/**
	 * Returns a copy of this model state with only the bits that matter for
	 * geometry. Used as lookup key for block damage models.
	 */
	MutableModelState geometricState();

	void toTag(class_2487 tag);

	void toBytes(class_2540 pBuff);

	default class_2487 toTag() {
		final class_2487 result = new class_2487();
		toTag(result);
		return result;
	}

	/**
	 * Output polygons must be quads or tris. Consumer MUST NOT hold references to
	 * any of the polys received.
	 */
	void emitPolygons(Consumer<Polygon> target);

	@Environment(EnvType.CLIENT)
	List<class_777> bakedQuads(class_2680 state, class_2350 face, Random rand);

	@Environment(EnvType.CLIENT)
	class_1058 particleSprite();

	@Environment(EnvType.CLIENT)
	int particleColorARBG();

	@Environment(EnvType.CLIENT)
	void renderAsBlock(BlockInputContext input, QuadSink output);

	@Environment(EnvType.CLIENT)
	void renderAsItem(ItemInputContext input, QuadSink output);

	@Environment(EnvType.CLIENT)
	class_1087 itemProxy();

	static MutableModelState fromTag(class_2487 tag, PaintIndex paintIndex) {
		return ModelPrimitiveRegistryImpl.INSTANCE.fromTag(tag, paintIndex);
	}

	static MutableModelState fromBytes(class_2540 pBuff, PaintIndex paintIndex) {
		return ModelPrimitiveRegistryImpl.INSTANCE.fromBytes(pBuff, paintIndex);
	}
}
