/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.api.paint;

import grondag.xm.paint.PaintIndexImpl;
import net.minecraft.class_1937;

public interface PaintIndex {
	XmPaint fromIndex(int paintIndex);

	/**
	 * Will throw an unsupported operation exception if called on client.
	 *
	 * <p>Persists paint with the world save and returns an index that can
	 * later be used to retrieve an anonymous paint instance. These
	 * indexes are not discoverable via the index, and should be
	 * saved and/or organized somehow by the consuming mod.

	 * @param paint to be indexed
	 * @return identical paint with world-persistent index
	 */
	XmPaint index(XmPaint paint);

	/**
	 * Remaps the paint associated with the given index.
	 * Throws an exception if the index does not exist.
	 * @param index
	 * @param paint
	 */
	void updateIndex(int index, XmPaint paint);

	static PaintIndex forWorld(class_1937 world) {
		return world == null || world.field_9236 ? PaintIndexImpl.CLIENT : PaintIndexImpl.SERVER;
	}
}
