/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.api.primitive;

import java.util.function.Function;
import net.minecraft.class_2960;
import org.jetbrains.annotations.ApiStatus.Experimental;
import grondag.fermion.orientation.api.OrientationType;
import grondag.xm.api.mesh.XmMesh;
import grondag.xm.api.modelstate.primitive.MutablePrimitiveState;
import grondag.xm.api.modelstate.primitive.PrimitiveState;
import grondag.xm.api.primitive.surface.XmSurfaceList;
import grondag.xm.primitive.SimplePrimitiveBuilderImpl;

@Experimental
public interface SimplePrimitive extends ModelPrimitive<PrimitiveState, MutablePrimitiveState> {
	static Builder builder() {
		return SimplePrimitiveBuilderImpl.builder();
	}

	public interface Builder {
		default SimplePrimitive build(class_2960 id) {
			return build(id);
		}

		Builder surfaceList(XmSurfaceList list);

		Builder primitiveBitCount(int bitCount);

		Builder orientationType(OrientationType orientationType);

		Builder polyFactory(Function<PrimitiveState, XmMesh> polyFactory);

		Builder simpleJoin(boolean needsJoin);

		Builder cornerJoin(boolean needsJoin);

		Builder axisJoin(boolean needsJoin);

		Builder alternateJoinAffectsGeometry(boolean alternateJoinAffectsGeometry);
	}
}
