/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.collision;

import org.jetbrains.annotations.ApiStatus.Internal;
import io.vram.sc.cache.ObjectSimpleLoadingCache;
import net.minecraft.class_259;
import net.minecraft.class_265;
import grondag.xm.api.modelstate.ModelState;

@Internal
public class CollisionDispatcherImpl {
	private static final ObjectSimpleLoadingCache<ModelState, class_265> modelCache = new ObjectSimpleLoadingCache<>(
			CollisionDispatcherImpl::load, k -> k.toImmutable(), 0xFFF);

	private static final ObjectSimpleLoadingCache<VoxelVolumeKey, class_265> volCache = new ObjectSimpleLoadingCache<>(
			k -> k.build(), k -> k.toImmutable(), 0xFFF);

	private static ThreadLocal<MeshVoxelizer> fastBoxGen = new ThreadLocal<>() {
		@Override
		protected MeshVoxelizer initialValue() {
			return new MeshVoxelizer();
		}
	};

	public static class_265 shapeFor(ModelState modelState) {
		return modelState == null ? class_259.method_1077() : modelCache.get(modelState.geometricState());
	}

	/**
	 * Clears the cache.
	 */
	public static void clear() {
		modelCache.clear();
		volCache.clear();
	}

	private static class_265 load(ModelState key) {
		final MeshVoxelizer generator = fastBoxGen.get();
		key.emitPolygons(generator);

		final VoxelVolumeKey vKey = generator.build();

		return volCache.get(vKey);
	}
}
