/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.dispatch;

import java.util.function.BiFunction;
import java.util.function.Function;
import net.minecraft.class_1747;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2680;
import org.jetbrains.annotations.ApiStatus.Internal;
import grondag.xm.Xm;
import grondag.xm.api.block.XmBlockState;
import grondag.xm.api.modelstate.ModelStateFunction;
import grondag.xm.api.modelstate.MutableModelState;

@Internal
public class XmRegistryImpl {
	private XmRegistryImpl() {
	}

	public static void register(class_2248 block, Function<class_2680, ? extends ModelStateFunction<?>> modelFunctionMap, BiFunction<class_1799, class_1937, MutableModelState> itemModelFunction) {
		for (final class_2680 blockState : block.method_9595().method_11662()) {
			if (XmBlockState.get(blockState) != null) {
				// TODO: localize
				Xm.LOG.warn(String.format("[%s] BlockState %s already associated with an XmBlockState. Skipping.", Xm.MODID, blockState.toString()));
				return;
			}

			((XmBlockStateAccess) blockState).xm_modelStateFunc(modelFunctionMap.apply(blockState));

			if (itemModelFunction != null) {
				final class_1792 item = class_1747.method_7867(block);

				if (item != null) {
					register(item, itemModelFunction);
				}
			}
		}
	}

	public static void register(class_1792 item, BiFunction<class_1799, class_1937, MutableModelState> modelFunction) {
		final XmItemAccess access = (XmItemAccess) item;
		final BiFunction<class_1799, class_1937, MutableModelState> oldFunc = access.xm_modelStateFunc();

		if (oldFunc != null) {
			if (oldFunc != modelFunction) {
				Xm.LOG.warn(String.format("[%s] Item %s already associated with a model function. Skipping.", Xm.MODID, item.toString()));
				return;
			}
		}

		access.xm_modelStateFunc(modelFunction);
	}
}
