/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.render;

import org.jetbrains.annotations.ApiStatus.Internal;
import grondag.xm.api.mesh.WritableMesh;
import grondag.xm.api.mesh.XmMeshes;
import grondag.xm.api.modelstate.ModelState;
import net.minecraft.class_1159;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_287;
import net.minecraft.class_4581;
import net.minecraft.class_4587;
import net.minecraft.class_4588;

@Internal
public class OutlineRenderer {
	private static ModelState outlineModelState = null;
	private static final WritableMesh outlineMesh = XmMeshes.claimWritable();

	/**
	 * Draws block-aligned grid on sides of AABB if entity can see it from outside.
	 */

	public static void drawModelOutline(class_4587 matrixStack, class_4588 vertexConsumer, ModelState modelState, double x, double y, double z, float r, float g, float b, float a) {
		if (modelState == null) {
			return;
		}

		final WritableMesh mesh = outlineMesh;
		final class_1159 matrix4f = matrixStack.method_23760().method_23761();
		final class_4581 normalMatrix = matrixStack.method_23760().method_23762();

		if (outlineModelState == null || !modelState.equals(outlineModelState)) {
			outlineModelState = modelState.toImmutable();
			mesh.clear();
			outlineModelState.emitPolygons(p -> mesh.appendCopy(p));
		}

		mesh.forEach(p -> {
			final int limit = p.vertexCount() - 1;

			for (int i = 0; i < limit; i++) {
				// lines format wants a normal - god knows what for - we always pass in up
				vertexConsumer.method_22918(matrix4f, (float) (p.x(i) + x), (float) (p.y(i) + y), (float) (p.z(i) + z)).method_22915(r, g, b, a).method_23763(normalMatrix, 0, 1, 0).method_1344();
				vertexConsumer.method_22918(matrix4f, (float) (p.x(i + 1) + x), (float) (p.y(i + 1) + y), (float) (p.z(i + 1) + z)).method_22915(r, g, b, a).method_23763(normalMatrix, 0, 1, 0).method_1344();
			}
		});
	}

	@SuppressWarnings("unused")
	public static void drawGrid(class_287 buffer, class_238 aabb, class_243 viewFrom, double offsetX, double offsetY, double offsetZ, float red, float green,
			float blue, float alpha) {
		final double minX = aabb.field_1323 - offsetX;
		final double minY = aabb.field_1322 - offsetY;
		final double minZ = aabb.field_1321 - offsetZ;
		final double maxX = aabb.field_1320 - offsetX;
		final double maxY = aabb.field_1325 - offsetY;
		final double maxZ = aabb.field_1324 - offsetZ;
		final int xSpan = (int) (aabb.field_1320 + 0.0001 - aabb.field_1323);
		final int ySpan = (int) (aabb.field_1325 + 0.0001 - aabb.field_1322);
		final int zSpan = (int) (aabb.field_1324 + 0.0001 - aabb.field_1321);

		//TODO: reimplement for new rendering system
		//        if (xSpan > 1 && zSpan > 1) {
		//            double dy = viewFrom.y > aabb.maxY ? maxY : viewFrom.y < aabb.minY ? minY : Double.MAX_VALUE;
		//            if (dy != Double.MAX_VALUE) {
		//                for (int x = 1; x <= xSpan; x++) {
		//                    double dx = minX + x;
		//                    buffer.vertex(dx, dy, minZ).color(red, green, blue, alpha).end();
		//                    buffer.vertex(dx, dy, maxZ).color(red, green, blue, alpha).end();
		//                }
		//                for (int z = 1; z <= zSpan; z++) {
		//                    double dz = minZ + z;
		//                    buffer.vertex(minX, dy, dz).color(red, green, blue, alpha).end();
		//                    buffer.vertex(maxX, dy, dz).color(red, green, blue, alpha).end();
		//                }
		//            }
		//        }
		//
		//        if (ySpan > 1) {
		//            if (zSpan > 1) {
		//                double dx = viewFrom.x > aabb.maxX ? maxX : viewFrom.x < aabb.minX ? minX : Double.MAX_VALUE;
		//                if (dx != Double.MAX_VALUE) {
		//                    for (int y = 1; y <= ySpan; y++) {
		//                        double dy = minY + y;
		//                        buffer.vertex(dx, dy, minZ).color(red, green, blue, alpha).end();
		//                        buffer.vertex(dx, dy, maxZ).color(red, green, blue, alpha).end();
		//                    }
		//                    for (int z = 1; z <= zSpan; z++) {
		//                        double dz = minZ + z;
		//                        buffer.vertex(dx, minY, dz).color(red, green, blue, alpha).end();
		//                        buffer.vertex(dx, maxY, dz).color(red, green, blue, alpha).end();
		//                    }
		//                }
		//            }
		//
		//            if (xSpan > 1) {
		//                double dz = viewFrom.z > aabb.maxZ ? maxZ : viewFrom.z < aabb.minZ ? minZ : Double.MAX_VALUE;
		//                if (dz != Double.MAX_VALUE) {
		//                    for (int y = 1; y <= ySpan; y++) {
		//                        double dy = minY + y;
		//                        buffer.vertex(minX, dy, dz).color(red, green, blue, alpha).end();
		//                        buffer.vertex(maxX, dy, dz).color(red, green, blue, alpha).end();
		//                    }
		//                    for (int x = 1; x <= xSpan; x++) {
		//                        double dx = minX + x;
		//                        buffer.vertex(dx, minY, dz).color(red, green, blue, alpha).end();
		//                        buffer.vertex(dx, maxY, dz).color(red, green, blue, alpha).end();
		//                    }
		//                }
		//            }
		//        }
	}
}
