/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.terrain;

import java.util.HashMap;
import net.minecraft.class_2248;
import com.google.common.collect.HashBiMap;
import org.jetbrains.annotations.ApiStatus.Internal;

/** Tracks which terrain blocks can be frozen or thawed from each other. */
@Internal
public class TerrainBlockRegistry {
	private final HashBiMap<class_2248, class_2248> stateMap = HashBiMap.create(16);
	private final HashBiMap<class_2248, class_2248> fillerMap = HashBiMap.create(16);
	private final HashMap<class_2248, class_2248> cubicMap = new HashMap<>(16);
	public static final TerrainBlockRegistry TERRAIN_STATE_REGISTRY = new TerrainBlockRegistry();

	public void registerStateTransition(class_2248 dynamicBlock, class_2248 staticBlock) {
		stateMap.put(dynamicBlock, staticBlock);
	}

	public TerrainStaticBlock getStaticBlock(class_2248 dynamicBlock) {
		return (TerrainStaticBlock) stateMap.get(dynamicBlock);
	}

	public class_2248 getDynamicBlock(class_2248 staticBlock) {
		return stateMap.inverse().get(staticBlock);
	}

	public void registerFiller(class_2248 heightBlock, class_2248 fillerBlock) {
		fillerMap.put(heightBlock, fillerBlock);
	}

	public class_2248 getFillerBlock(class_2248 hieghtBlock) {
		return fillerMap.get(hieghtBlock);
	}

	public class_2248 getHeightBlock(class_2248 fillerBlock) {
		return fillerMap.inverse().get(fillerBlock);
	}

	public void registerCubic(class_2248 flowBlock, class_2248 cubicBlock) {
		cubicMap.put(flowBlock, cubicBlock);
	}

	public class_2248 getCubicBlock(class_2248 flowBlock) {
		return cubicMap.get(flowBlock);
	}
}
