/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.terrain;

import org.jetbrains.annotations.ApiStatus.Internal;
import grondag.xm.api.modelstate.ModelState;
import grondag.xm.api.modelstate.MutableModelState;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2591;
import net.minecraft.class_2680;

@Internal
public class TerrainStaticBlock extends class_2248 implements IHotBlock {
	public TerrainStaticBlock(class_2251 blockSettings, ModelState defaultModelState, class_2591<?> blockEntityType, boolean isFiller) {
		super(blockSettings); //, adjustShape(defaultModelState, isFiller), blockEntityType);
	}

	// PERF: sucks
	@SuppressWarnings("unused")
	private static MutableModelState adjustShape(ModelState stateIn, boolean isFiller) {
		final MutableModelState result = isFiller ? TerrainSurface.FILLER.newState() : TerrainSurface.HEIGHT.newState();
		result.copyFrom(stateIn);
		result.setStatic(true);
		return result;
	}

	//  TODO: remove or restore
	//    @Override
	//    public int quantityDropped(ExtendedBlockView world, BlockPos pos, BlockState state) {
	//        double volume = 0;
	//        ISuperModelState modelState = SuperBlockWorldAccess.access(world).computeModelState(this, state, pos, true);
	//        for (BoundingBox box : modelState.getShape().meshFactory().collisionHandler().getCollisionBoxes(modelState)) {
	//            volume += Useful.volumeAABB(box);
	//        }
	//
	//        return (int) Math.min(9, volume * 9);
	//    }

	/**
	 * Prevent neighboring dynamic blocks from updating geometry by making them
	 * static.
	 */
	@Override
	public void method_9576(class_1937 world, class_2338 pos, class_2680 state, class_1657 player) {
		TerrainDynamicBlock.freezeNeighbors(world, pos, state);
		super.method_9576(world, pos, state, player);
	}

	/**
	 * Prevent neighboring dynamic blocks from updating geometry by making them
	 * static.
	 */
	@Override
	public void method_9567(class_1937 worldIn, class_2338 pos, class_2680 state, class_1309 placer, class_1799 stack) {
		super.method_9567(worldIn, pos, state, placer, stack);
		TerrainDynamicBlock.freezeNeighbors(worldIn, pos, state);
	}

	/**
	 * Convert this block to a dynamic version of itself if one is known.
	 */
	public void makeDynamic(class_2680 state, class_1937 world, class_2338 pos) {
		final class_2680 newState = dynamicState(state, world, pos);

		if (newState != state) {
			world.method_8652(pos, newState, 3);
		}
	}

	/**
	 * Returns dynamic version of self if one is known. Otherwise returns self.
	 */
	public class_2680 dynamicState(class_2680 state, class_1922 world, class_2338 pos) {
		final class_2248 dynamicVersion = TerrainBlockRegistry.TERRAIN_STATE_REGISTRY.getDynamicBlock(this);

		if (dynamicVersion == null || state.method_26204() != this) {
			return state;
		}

		// TODO: transfer heat block state?
		return dynamicVersion.method_9564().method_11657(TerrainBlock.TERRAIN_TYPE, state.method_11654(TerrainBlock.TERRAIN_TYPE));
	}

	public void setModelState(class_1937 world, class_2338 pos, MutableModelState myModelState) {
		// TODO Auto-generated method stub
	}
}
