/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.virtual;

import org.jetbrains.annotations.ApiStatus.Internal;
import org.jetbrains.annotations.Nullable;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1297;
import net.minecraft.class_2338;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_287;
import net.minecraft.class_761;
import grondag.xm.Xm;
import grondag.xm.XmConfig;
import grondag.xm.render.OutlineRenderer;

@Internal
public class ExcavationRenderer {
	public final int id;

	/**
	 * If true, is replacement instead of straight excavation.
	 */
	public final boolean isExchange;

	private class_238 aabb;

	private class_238 visibilityBounds;

	private boolean didDrawBoundsLastTime = false;

	private class_243 lastEyePosition;

	/**
	 * If non-null, then we should render individual positions instead of AABB.
	 */
	private class_2338[] positions;

	public ExcavationRenderer(int id, class_238 aabb, boolean isExchange, @Nullable class_2338[] positions) {
		this.id = id;
		this.isExchange = isExchange;
		setBounds(aabb, positions);
	}

	public void setBounds(class_238 bounds, @Nullable class_2338[] positions) {
		aabb = bounds;
		visibilityBounds = bounds.method_1014(192);
		this.positions = positions;

		if (XmConfig.logExcavationRenderTracking) {
			Xm.LOG.info("id %d Renderer setBounds position count = %d", id, positions == null ? 0 : positions.length);
		}
	}

	public class_238 bounds() {
		return aabb;
	}

	public class_238 visibilityBounds() {
		return visibilityBounds;
	}

	/** Return true if something was drawn. */
	@Environment(EnvType.CLIENT)
	public boolean drawBounds(class_287 bufferbuilder, class_1297 viewEntity, double d0, double d1, double d2, float partialTicks) {
		lastEyePosition = viewEntity.method_5836(partialTicks);

		if (visibilityBounds.method_1006(lastEyePosition)) {
			if (positions == null) {
				final class_238 box = aabb;
				class_761.method_3258(bufferbuilder, box.field_1323 - d0, box.field_1322 - d1, box.field_1321 - d2, box.field_1320 - d0, box.field_1325 - d1, box.field_1324 - d2, 1f, 0.3f, 0.3f,
					1f);
			} else {
				for (final class_2338 pos : positions) {
					final double x = pos.method_10263() - d0;
					final double y = pos.method_10264() - d1;
					final double z = pos.method_10260() - d2;
					class_761.method_3258(bufferbuilder, x, y, z, x + 1, y + 1, z + 1, 1f, 0.3f, 0.3f, 1f);
				}
			}

			didDrawBoundsLastTime = true;
			return true;
		} else {
			didDrawBoundsLastTime = false;
			return false;
		}
	}

	@Environment(EnvType.CLIENT)
	public void drawGrid(class_287 buffer, double d0, double d1, double d2) {
		if (didDrawBoundsLastTime && positions == null) {
			OutlineRenderer.drawGrid(buffer, aabb, lastEyePosition, d0, d1, d2, 1f, 0.3f, 0.3f, 0.5F);
		}
	}

	@Environment(EnvType.CLIENT)
	public void drawBox(class_287 bufferbuilder, double d0, double d1, double d2) {
		if (didDrawBoundsLastTime) {
			if (positions == null) {
				final class_238 box = aabb;
				class_761.method_3258(bufferbuilder, box.field_1323 - d0, box.field_1322 - d1, box.field_1321 - d2, box.field_1320 - d0, box.field_1325 - d1, box.field_1324 - d2, 1f, 0.3f,
					0.3f, 0.3f);
			} else {
				for (final class_2338 pos : positions) {
					final double x = pos.method_10263() - d0;
					final double y = pos.method_10264() - d1;
					final double z = pos.method_10260() - d2;
					class_761.method_3258(bufferbuilder, x, y, z, x + 1, y + 1, z + 1, 1f, 0.3f, 0.3f, 0.3f);
				}
			}
		}
	}
}
