/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.virtual;

import org.jetbrains.annotations.ApiStatus.Internal;
import grondag.xm.api.modelstate.ModelState;
import net.minecraft.class_1657;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2680;

@Internal
public interface VirtualBlock {
	/**
	 * True if block at the given position is actually solid (not replaceable) or is
	 * virtual and visible to the given player.
	 */
	default boolean isVirtuallySolid(class_2338 pos, class_1657 player) {
		return !player.field_6002.method_8320(pos).method_26207().method_15800();
	}

	/**
	 * Convenience call for {@link #isVirtual()} when you don't know what type of
	 * block it is. Will return false for any block that doesn't implement
	 * ISuperBlock.
	 */
	static boolean isVirtualBlock(class_2248 block) {
		return block instanceof VirtualBlock;
	}

	/**
	 * Convenience call for
	 * {@link #isVirtuallySolid(ExtendedBlockView, BlockPos, EntityPlayer)} when you
	 * don't know what type of block it is. Will return false for any block that
	 * doesn't implement ISuperBlock.
	 */
	static boolean isVirtuallySolidBlock(class_2338 pos, class_1657 player) {
		return isVirtuallySolidBlock(player.field_6002.method_8320(pos), pos, player);
	}

	/**
	 * Convenience call for
	 * {@link #isVirtuallySolid(ExtendedBlockView, BlockPos, EntityPlayer)} when you
	 * don't know what type of block it is. Will return negation of
	 * {@link #isReplaceable(ExtendedBlockView, BlockPos)} for any block that
	 * doesn't implement ISuperBlock.
	 */
	// UGLY: really needed? Seems redundant of isVirtuallySolid, plus have mixins  now
	static boolean isVirtuallySolidBlock(class_2680 state, class_2338 pos, class_1657 player) {
		final class_2248 block = state.method_26204();
		return isVirtualBlock(block) ? ((VirtualBlock) block).isVirtuallySolid(pos, player) : !state.method_26207().method_15800();
	}

	static class_2680 findAppropriateVirtualBlock(ModelState modelState) {
		// TODO Auto-generated method stub
		return null;
	}
}
